@@ -19,6 +19,7 @@ import com.unciv.models.ruleset.Ruleset
 import com.unciv.models.ruleset.RulesetCache
 import com.unciv.models.ruleset.unique.StateForConditionals
 import com.unciv.models.ruleset.unique.UniqueType
+import com.unciv.models.ruleset.unique.UniqueTriggerActivation
 import com.unciv.models.ruleset.unit.BaseUnit
 import com.unciv.models.stats.Stats
 import com.unciv.models.translations.equalsPlaceholderText
@@ -167,9 +168,14 @@ object GameStarter {
     private fun addCivTechs(gameInfo: GameInfo, ruleset: Ruleset, gameSetupInfo: GameSetupInfo) {
         for (civInfo in gameInfo.civilizations.filter { !it.isBarbarian() }) {
 
+        for(tech in ruleset.technologies.values.filter { it.hasUnique(UniqueType.StartingTech) })
+            {
+                civInfo.tech.addTechnology(tech.name, false)
+            }
+
             if (!civInfo.isHuman())
                 for (tech in gameInfo.getDifficulty().aiFreeTechs)
-                    civInfo.tech.addTechnology(tech)
+                    civInfo.tech.addTechnology(tech, false)
 
             // generic start with technology unique
             for (unique in civInfo.getMatchingUniques(UniqueType.StartsWithTech)) {
@@ -178,19 +184,19 @@ object GameStarter {
 
                 // check if the technology is in the ruleset and not already researched
                 if (ruleset.technologies.containsKey(techName) && !civInfo.tech.isResearched(techName))
-                    civInfo.tech.addTechnology(techName)
+                    civInfo.tech.addTechnology(techName, false)
             }
 
             // add all techs to spectators
             if (civInfo.isSpectator())
                 for (tech in ruleset.technologies.values)
                     if (!civInfo.tech.isResearched(tech.name))
-                        civInfo.tech.addTechnology(tech.name)
+                        civInfo.tech.addTechnology(tech.name, false)
 
             for (tech in ruleset.technologies.values
                     .filter { ruleset.eras[it.era()]!!.eraNumber < ruleset.eras[gameSetupInfo.gameParameters.startingEra]!!.eraNumber })
                 if (!civInfo.tech.isResearched(tech.name))
-                    civInfo.tech.addTechnology(tech.name)
+                    civInfo.tech.addTechnology(tech.name, false)
 
             civInfo.popupAlerts.clear() // Since adding technologies generates popups...
         }
@@ -330,8 +336,6 @@ object GameStarter {
         ruleset: Ruleset,
         chosenPlayers: List<Player>
     ) {
-        val startingTechs = ruleset.technologies.values.filter { it.hasUnique(UniqueType.StartingTech) }
-
         if (!newGameParameters.noBarbarians && ruleset.nations.containsKey(Constants.barbarians)) {
             val barbarianCivilization = Civilization(Constants.barbarians)
             gameInfo.civilizations.add(barbarianCivilization)
@@ -349,8 +353,6 @@ object GameStarter {
                 Constants.spectator ->
                     civ.playerType = player.playerType
                 in usedMajorCivs -> {
-                    for (tech in startingTechs)
-                        civ.tech.techsResearched.add(tech.name) // can't be .addTechnology because the civInfo isn't assigned yet
                     civ.playerType = player.playerType
                     civ.playerId = player.playerId
                 }
@@ -402,7 +404,7 @@ object GameStarter {
             if (tile.improvement != null
                 && tile.getTileImprovement()!!.isAncientRuinsEquivalent()
             ) {
-                tile.changeImprovement(null) // Remove ancient ruins in immediate vicinity
+                tile.removeImprovement() // Remove ancient ruins in immediate vicinity
             }
         }
     }
@@ -422,11 +424,18 @@ object GameStarter {
             val startingUnits = getStartingUnitsForEraAndDifficulty(civ, gameInfo, ruleset, startingEra)
             adjustStartingUnitsForCityStatesAndOneCityChallenge(civ, gameInfo, startingUnits, settlerLikeUnits)
             placeStartingUnits(civ, startingLocation, startingUnits, ruleset, ruleset.eras[startingEra]!!.startingMilitaryUnit, settlerLikeUnits)
+
+            //Trigger any global or nation uniques that should triggered.
+            //We may need the starting location for some uniques, which is why we're doing it now
+            val startingTriggers = (ruleset.globalUniques.uniqueObjects + civ.nation.uniqueObjects)
+            for (unique in startingTriggers.filter { !it.hasTriggerConditional() })
+                if(unique.isTriggerable)
+                    UniqueTriggerActivation.triggerCivwideUnique(unique, civ, tile = startingLocation)
         }
     }
 
     private fun getStartingUnitsForEraAndDifficulty(civ: Civilization, gameInfo: GameInfo, ruleset: Ruleset, startingEra: String): MutableList<String> {
-        val startingUnits = ruleset.eras[startingEra]!!.getStartingUnits().toMutableList()
+        val startingUnits = ruleset.eras[startingEra]!!.getStartingUnits(ruleset)
 
         // Add extra units granted by difficulty
         startingUnits.addAll(when {
