@@ -79,7 +79,7 @@ object UnitAutomation {
 
         // If everything around this unit is visible, we can stop.
         // Calculations below are quite expensive especially in the late game.
-        if (unit.currentTile.getTilesInDistance(5).any { !it.isVisible(unit.civ) }) {
+        if (unit.currentTile.getTilesInDistance(5).all { it.isVisible(unit.civ) }) {
             return false
         }
 
@@ -110,10 +110,14 @@ object UnitAutomation {
     }
 
     @JvmStatic
-    fun wander(unit: MapUnit, stayInTerritory: Boolean = false) {
+    fun wander(unit: MapUnit, stayInTerritory: Boolean = false, tilesToAvoid:Set<Tile> = setOf()) {
         val unitDistanceToTiles = unit.movement.getDistanceToTiles()
         val reachableTiles = unitDistanceToTiles
-                .filter { unit.movement.canMoveTo(it.key) && unit.movement.canReach(it.key) }
+                .filter {
+                    it.key !in tilesToAvoid
+                    && unit.movement.canMoveTo(it.key)
+                    && unit.movement.canReach(it.key)
+                }
 
         val reachableTilesMaxWalkingDistance = reachableTiles
                 .filter { it.value.totalDistance == unit.currentMovement
@@ -234,11 +238,24 @@ object UnitAutomation {
                 unit.movement.moveToTile(tilesCanMoveTo.minByOrNull { it.value.totalDistance }!!.key)
         }
 
+        val tilesWhereWeWillBeCaptured = unit.currentTile.getTilesInDistance(5)
+            .mapNotNull { it.militaryUnit }
+            .filter { it.civ.isAtWarWith(unit.civ) }
+            .flatMap { it.movement.getReachableTilesInCurrentTurn() }
+            .filter { it.militaryUnit?.civ != unit.civ }
+            .toSet()
+
         if (unit.hasUnique(UniqueType.FoundCity))
-            return SpecificUnitAutomation.automateSettlerActions(unit)
+            return SpecificUnitAutomation.automateSettlerActions(unit, tilesWhereWeWillBeCaptured)
 
         if (unit.cache.hasUniqueToBuildImprovements)
-            return WorkerAutomation.automateWorkerAction(unit)
+            return unit.civ.getWorkerAutomation().automateWorkerAction(unit, tilesWhereWeWillBeCaptured)
+
+        if (unit.cache.hasUniqueToCreateWaterImprovements){
+            if (!unit.civ.getWorkerAutomation().automateWorkBoats(unit))
+                tryExplore(unit)
+            return
+        }
 
         if (unit.hasUnique(UniqueType.MayFoundReligion)
                 && unit.civ.religionManager.religionState < ReligionState.Religion
@@ -252,9 +269,6 @@ object UnitAutomation {
         )
             return SpecificUnitAutomation.enhanceReligion(unit)
 
-        if (unit.hasUnique(UniqueType.CreateWaterImprovements))
-            return SpecificUnitAutomation.automateWorkBoats(unit)
-
         // We try to add any unit in the capital we can, though that might not always be desirable
         // For now its a simple option to allow AI to win a science victory again
         if (unit.hasUnique(UniqueType.AddInCapital))
@@ -314,8 +328,7 @@ object UnitAutomation {
 
         // This has to come after the individual abilities for the great people that can also place
         // instant improvements (e.g. great scientist).
-        if (unit.hasUnique(UniqueType.ConstructImprovementConsumingUnit)
-                || unit.hasUnique(UniqueType.ConstructImprovementInstantly)
+        if (unit.hasUnique(UniqueType.ConstructImprovementInstantly)
         ) {
             // catch great prophet for civs who can't found/enhance/spread religion
             // includes great people plus moddable units
@@ -683,7 +696,7 @@ object UnitAutomation {
     }
 
     private fun tryTakeBackCapturedCity(unit: MapUnit): Boolean {
-        var capturedCities = unit.civ.getKnownCivs().asSequence()
+        var capturedCities = unit.civ.getKnownCivs() // This is a Sequence
                 .flatMap { it.cities.asSequence() }
                 .filter {
                     unit.civ.isAtWarWith(it.civ) &&
@@ -762,14 +775,12 @@ object UnitAutomation {
     private fun tryRunAwayIfNeccessary(unit: MapUnit): Boolean {
         // This is a little 'Bugblatter Beast of Traal': Run if we can attack an enemy
         // Cheaper than determining which enemies could attack us next turn
-        //todo - stay when we're stacked with a good military unit???
         val enemyUnitsInWalkingDistance = unit.movement.getDistanceToTiles().keys
             .filter { containsEnemyMilitaryUnit(unit, it) }
 
-        if (enemyUnitsInWalkingDistance.isNotEmpty() && !unit.baseUnit.isMilitary()) {
-            if (unit.getTile().militaryUnit == null && !unit.getTile().isCityCenter())
-                runAway(unit)
-
+        if (enemyUnitsInWalkingDistance.isNotEmpty() && !unit.baseUnit.isMilitary()
+            && unit.getTile().militaryUnit == null && !unit.getTile().isCityCenter()) {
+            runAway(unit)
             return true
         }
 
